(* fichier comptage.ml: recherche par comptage *)

(* exposants de 2,3,5 et logarithme *)
type entier = {a:int; b:int; c:int; l:float};;
let lg3 = log(3.0)/.log(2.0)
and lg5 = log(5.0)/.log(2.0)

(* cherche le plus petit k tel que n(2^k) >= n *)
(* retourne n(2^k)-n et k                      *)
let rec cherche n k =
  if n <= 0 then (-n,k)
  else begin
    let w = ref(float(k+1))
    and c = ref(n) in
    while !w >= 0.0 do c := !c - truncate(!w/.lg3) - 1; w := !w -. lg5 done;
    cherche !c (k+1)
  end

(* liste trie des points dans la tranche ]2^(k-1),2^k] *)
let liste(k) =
  let res = ref [] in
  for z=0 to truncate(float(k)/.lg5) do
    for y=0 to truncate((float(k)-.float(z)*.lg5)/.lg3) do
      let l = float(y)*.lg3 +. float(z)*.lg5 in
      let x = truncate(float(k) -. l)        in
      res := {a=x; b=y; c=z; l = float(x) +. l} :: !res
    done
  done;
  Sort.list (fun u v -> u.l > v.l) !res
  

(* lance les calculs *)
let _ =
  let n = int_of_string(Sys.argv.(1)) in
  let (p,k) = cherche n 0      in
  let t = List.nth (liste k) p in
  Printf.printf "2^%d . 3^%d . 5^%d" t.a t.b t.c;
  print_newline()
