(* fichier prifile.ml : insertion dans une file de priorit *)

(* reprsentation de 2^a*3^b*5^c, l=logarithme *)
type entier = {a:int; b:int; c:int; l:float}
let ln2 = log(2.0)
and ln3 = log(3.0)
and ln5 = log(5.0)

(* file de priorit = ensemble ordonn *)
module Hset = Set.Make(
  struct
    type t = entier
    let compare x y = Pervasives.compare x.l y.l
  end)
open Hset

(* avance de n lments *)
let rec avance n h =
  let x = min_elt h in
  if n=0 then x
  else
    let h = remove x h in
    let h = add {x with c=x.c+1; l=x.l+.ln5} h in
    let h = if x.c=0         then add {x with b=x.b+1; l=x.l+.ln3} h else h in
    let h = if x.c=0 & x.b=0 then add {x with a=x.a+1; l=x.l+.ln2} h else h in
    avance (n-1) h

(* lance les calculs *)
let _ =
  let h = singleton {a=0; b=0; c=0; l=0.} in
  let n = int_of_string(Sys.argv.(1)) in
  let t = avance n h in
  Printf.printf "2^%d . 3^%d . 5^%d" t.a t.b t.c;
  print_newline()
